/* swlib - Smith-Waterman search library */
/* Copyright (C) 2001  OKUJI K. Yoshinori <o@bioruby.org>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.  */

#ifndef SW_HEADER
#define SW_HEADER	1


/* Error */

/* All swlib functions return this.  */
enum _sw_err_t
{
  SW_ERR_SUCCESS = 0,	/* succeed */
  SW_ERR_NOMEM,		/* no memory */
  SW_ERR_BADARG,	/* invalid argument */
  SW_ERR_BUG,		/* internal error */
};
typedef enum _sw_err_t sw_err_t;

/* Translate an error code to a string.  */
const char *sw_strerror (sw_err_t err);


/* Sequence */

/* Virtualize DNA/AA sequences.  */
struct _sw_seq_t
{
  char *data;
  int len;
};
typedef struct _sw_seq_t sw_seq_t;

/* Create a sequence object from a DNA/AA sequence.  */
sw_err_t sw_seq_init (sw_seq_t *seq, const char *data);

/* Append DATA to SEQ.  */
sw_err_t sw_seq_append (sw_seq_t *seq, const char *data);

/* Prepend DATA to SEQ.  */
sw_err_t sw_seq_prepend (sw_seq_t *seq, const char *data);

/* Create an AA sequence object from a DNA sequence. START is the beginning
   point to try to translate DNA_SEQ, and END is the end point. If you
   specify -1 as START, then this function tries to translate DNA_SEQ from
   the first candidate of possible start codons. If you specify END as less
   than or equal to START, then try to translate the whole sequence of
   DNA_SEQ. If START_CODON is non-zero, the translation begins with the
   first candidate of possible start codons, otherwise ignore any start
   codon.  */
sw_err_t sw_seq_translate (sw_seq_t *aa_seq, const sw_seq_t *dna_seq,
			   int start, int end, int start_codon);

/* Free the resource for SEQ.  */
sw_err_t sw_seq_free (sw_seq_t *seq);

#define sw_seq_data(seq)	((seq)->data)
#define sw_seq_len(seq)		((seq)->len)


/* Database */

/* The sequence database type.  */
struct _sw_db_t
{
  unsigned size;
  sw_seq_t seqs[0];
};
typedef struct _sw_db_t sw_db_t;

/* Initialize the database.  */
sw_err_t sw_db_init (sw_db_t *db);

/* Add SEQ to DB.  */
sw_err_t sw_db_add (sw_db_t *db, const sw_seq_t *seq);

/* Free the database.  */
sw_err_t sw_db_free (sw_db_t *db);

#define sw_db_size(db)	((db)->size)
#define sw_db_seq(db,n)	((db)->seqs + (n))


/* Alignment */

/* Alignment types.  */
enum _sw_align_type_t
{
  SW_ALIGN_MATCH,
  SW_ALIGN_MISMATCH,
  SW_ALIGN_Q_GAP,
  SW_ALIGN_T_GAP
};
typedef enum _sw_align_type_t sw_align_type_t;

/* Information about an alignment.  */
struct _sw_alignment_t
{
  int q_start;
  int q_end;
  int t_start;
  int t_end;
  int len;
  sw_align_type_t types[0];
};
typedef struct _sw_alignment_t sw_alignment_t;

#define sw_alignment_type(al,n)		((al)->types[n])
#define sw_alignment_q_start(al)	((al)->q_start)
#define sw_alignment_q_end(al)		((al)->q_end)
#define sw_alignment_q_len(al)		((al)->q_end - (al)->q_start + 1)
#define sw_alignment_t_start(al)	((al)->t_start)
#define sw_alignment_t_end(al)		((al)->t_end)
#define sw_alignment_t_len(al)		((al)->t_end - (al)->t_start + 1)
#define sw_alignment_len(al)		((al)->len)


/* Computation */

/* Compute the Smith-Waterman score between Q_SEQ and T_SEQ. The sequences
   must be DNA sequences. MATCH, MISMATCH, OPEN_GAP and EXTEND_GAP define
   the scoring system (e.g. MATCH = 5, MISMATCH = -4, OPEN_GAP = -16,
   EXTEND_GAP = -4). The best score is returned in *SCORE and the best
   alignment in *ALIGNMENT. You can pass NULL as SCORE or ALIGNMENT.  */
sw_err_t sw_comp_dna (const sw_seq_t *q_seq, const sw_seq_t *t_seq,
		      int match, int mismatch, int open_gap, int extend_gap,
		      int *score, sw_alignment_t **alignment);

#endif /* SW_HEADER */
