/* swlib - Smith-Waterman search library */
/* Copyright (C) 2001  OKUJI K. Yoshinori <o@bioruby.org>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.  */

#include <sw.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* ``cell'' is the fundamental object in a SW table.  */
enum _cell_state_t
{
  STATE_NONE = 0,
  STATE_NO_GAP,
  STATE_H_GAP,
  STATE_V_GAP
};
typedef enum _cell_state_t cell_state_t;

struct _cell_t
{
  int score;
  cell_state_t state;
};
typedef struct _cell_t cell_t;

/* Compute the Smith-Waterman score between Q_SEQ and T_SEQ. The sequences
   must be DNA sequences. MATCH, MISMATCH, OPEN_GAP and EXTEND_GAP define
   the scoring system (e.g. MATCH = 5, MISMATCH = -4, OPEN_GAP = -16,
   EXTEND_GAP = -4). The best score is returned in *SCORE and the best
   alignment in *ALIGNMENT. You can pass NULL as SCORE or ALIGNMENT.  */
sw_err_t
sw_comp_dna (const sw_seq_t *q_seq, const sw_seq_t *t_seq,
	     int match, int mismatch, int open_gap, int extend_gap,
	     int *score, sw_alignment_t **alignment)
{
  cell_t *nogap_table, *hgap_table, *vgap_table;
  unsigned size;
  unsigned i, j;
  cell_t *max_score_cell = 0;
  unsigned width, height;
  int offset;

  /* Sanity check.  */
  if (! q_seq || ! q_seq->data || ! t_seq || ! t_seq->data)
    return SW_ERR_BADARG;

  /* Allocate the SW table.  */
  width = sw_seq_len (q_seq) + 1;
  height = sw_seq_len (t_seq) + 1;
  size = width * height;
  nogap_table = malloc (size * sizeof (cell_t));
  hgap_table = malloc (size * sizeof (cell_t));
  vgap_table = malloc (size * sizeof (cell_t));
  if (! nogap_table || ! hgap_table || ! vgap_table)
    {
      if (nogap_table)
	free (nogap_table);
      if (hgap_table)
	free (hgap_table);
      return SW_ERR_NOMEM;
    }

  for (i = 0; i < width; i++)
    {
      nogap_table[i].state = STATE_NONE;
      nogap_table[i].score = 0;
      
      hgap_table[i].state = STATE_NONE;
      hgap_table[i].score = 0;

      vgap_table[i].state = STATE_NONE;
      vgap_table[i].score = 0;
    }
  for (i = width + 1; i < width * height; i += width)
    {
      nogap_table[i].state = STATE_NONE;
      nogap_table[i].score = 0;
      
      hgap_table[i].state = STATE_NONE;
      hgap_table[i].score = 0;

      vgap_table[i].state = STATE_NONE;
      vgap_table[i].score = 0;
    }      

  for (j = 0, offset = width + 1; j < sw_seq_len (t_seq); j++, offset++)
    for (i = 0; i < sw_seq_len (q_seq); i++, offset++)
      {
	int score;
	int nogap_score, hgap_score, vgap_score;
	int l_offset = offset - 1;
	int u_offset = offset - width;
	int ul_offset = u_offset -1;
	int max_score = 0;

	/* Calculate the no gap score.  */
	if (sw_seq_data (q_seq)[i] == sw_seq_data (t_seq)[j])
	  score = match;
	else
	  score = mismatch;

	nogap_score = nogap_table[ul_offset].score + score;
	hgap_score = hgap_table[ul_offset].score + score;
	vgap_score = vgap_table[ul_offset].score + score;
	if (nogap_score > 0
	    && nogap_score >= hgap_score
	    && nogap_score >= vgap_score)
	  {
	    max_score = nogap_score;
	    nogap_table[offset].score = max_score;
	    nogap_table[offset].state = STATE_NO_GAP;
	  }
	else if (hgap_score > 0 && hgap_score >= vgap_score)
	  {
	    max_score = hgap_score;
	    nogap_table[offset].score = max_score;
	    nogap_table[offset].state = STATE_H_GAP;
	  }
	else if (vgap_score > 0)
	  {
	    max_score = vgap_score;
	    nogap_table[offset].score = max_score;
	    nogap_table[offset].state = STATE_V_GAP;
	  }
	else
	  {
	    nogap_table[offset].score = 0;
	    nogap_table[offset].state = STATE_NONE;
	  }

	/* Check if the current cell has the max score.  */
	if (max_score
	    && (! max_score_cell || max_score > max_score_cell->score))
	  max_score_cell = &nogap_table[offset];

	/* Calculate the horizontal gap score.  */
	nogap_score = nogap_table[l_offset].score + open_gap;
	hgap_score = hgap_table[l_offset].score + extend_gap;
	if (nogap_score > 0
	    && nogap_score >= hgap_score)
	  {
	    hgap_table[offset].score = nogap_score;
	    hgap_table[offset].state = STATE_NO_GAP;
	  }
	else if (hgap_score > 0)
	  {
	    hgap_table[offset].score = hgap_score;
	    hgap_table[offset].state = STATE_H_GAP;
	  }
	else
	  {
	    hgap_table[offset].score = 0;
	    hgap_table[offset].state = STATE_NONE;
	  }

	/* Calculate the vertical gap score.  */
	nogap_score = nogap_table[u_offset].score + open_gap;
	vgap_score = vgap_table[u_offset].score + extend_gap;
	if (nogap_score > 0
	    && nogap_score >= vgap_score)
	  {
	    vgap_table[offset].score = nogap_score;
	    vgap_table[offset].state = STATE_NO_GAP;
	  }
	else if (vgap_score > 0)
	  {
	    vgap_table[offset].score = vgap_score;
	    vgap_table[offset].state = STATE_V_GAP;
	  }
	else
	  {
	    vgap_table[offset].score = 0;
	    vgap_table[offset].state = STATE_NONE;
	  }
      }

  /* Set *SCORE.  */
  if (score)
    *score = max_score_cell->score;
  
  /* Get the alignment.  */
  if (alignment)
    {
      if (! max_score_cell)
	{
	  *alignment = (sw_alignment_t *) malloc (sizeof (sw_alignment_t));
	  if (! *alignment)
	    {
	      free (nogap_table);
	      free (hgap_table);
	      free (vgap_table);
	      
	      return SW_ERR_NOMEM;
	    }
	  
	  (*alignment)->q_start = 0;
	  (*alignment)->q_end = -1;
	  (*alignment)->t_start = 0;
	  (*alignment)->t_end = -1;
	  (*alignment)->len = 0;
	}
      else
	{
	  int len = 0;
	  int q_len = 0;
	  int t_len = 0;
	  int max_len;
	  cell_t *back_cell;
	  cell_t *cur_cell;
	  cell_state_t state;
	  int offset;
	  sw_align_type_t *t1, *t2;
	  
	  /* At most, the length of the alignment is the sum of the
	     lengthes of the two sequences.  */
	  max_len = sw_seq_len (q_seq) + sw_seq_len (t_seq);

	  *alignment = (sw_alignment_t *)
	    malloc (sizeof (sw_alignment_t)
		    + sizeof (sw_align_type_t) * max_len);
	  if (! *alignment)
	    {
	      free (nogap_table);
	      free (hgap_table);
	      free (vgap_table);
	      
	      return SW_ERR_NOMEM;
	    }
	  
	  offset = max_score_cell - nogap_table;
	  i = (offset % width) - 1;
	  j = (offset / width) - 1;
	  cur_cell = max_score_cell;
	  state = STATE_NO_GAP;
	  while (1)
	    {
	      switch (state)
		{
		case STATE_NO_GAP:
		  if (sw_seq_data (q_seq)[i] == sw_seq_data (t_seq)[j])
		    (*alignment)->types[len] = SW_ALIGN_MATCH;
		  else
		    (*alignment)->types[len] = SW_ALIGN_MISMATCH;

		  i--;
		  j--;
		  q_len++;
		  t_len++;
		  break;
		  
		case STATE_H_GAP:
		  (*alignment)->types[len] = SW_ALIGN_T_GAP;
		  i--;
		  q_len++;
		  break;

		case STATE_V_GAP:
		  (*alignment)->types[len] = SW_ALIGN_Q_GAP;
		  j--;
		  t_len++;
		  break;

		default:
		  free (nogap_table);
		  free (hgap_table);
		  free (vgap_table);
		  
		  return SW_ERR_BUG;
		}

	      len++;
	      offset = (j + 1) * width + i + 1;

	      switch (cur_cell->state)
		{
		case STATE_NO_GAP:
		  back_cell = &nogap_table[offset];
		  break;

		case STATE_H_GAP:
		  back_cell = &hgap_table[offset];
		  break;
		  
		case STATE_V_GAP:
		  back_cell = &vgap_table[offset];
		  break;

		default:
		  free (nogap_table);
		  free (hgap_table);
		  free (vgap_table);

		  return SW_ERR_BUG;
		}
	      
	      if (back_cell->score == 0)
		break;
	      
	      state = cur_cell->state;
	      cur_cell = back_cell;
	    }

	  offset = cur_cell - nogap_table;
	  
	  (*alignment)->q_start = (offset % width) - 1;
	  (*alignment)->q_end = (*alignment)->q_start + q_len - 1;
	  (*alignment)->t_start = (offset / width) - 1;
	  (*alignment)->t_end = (*alignment)->t_start + t_len - 1;
	  (*alignment)->len = len;

	  /* Reverse the result, because the direction was backward.  */
	  t1 = (*alignment)->types;
	  t2 = t1 + len - 1;
	  while (t2 > t1)
	    {
	      sw_align_type_t tmp;

	      tmp = *t1;
	      *t1 = *t2;
	      *t2 = tmp;

	      t1++;
	      t2--;
	    }
	}
    }

  free (nogap_table);
  free (hgap_table);
  free (vgap_table);
  
  return SW_ERR_SUCCESS;
}

	  
	  

	  
